#include <DS1307.h> // Librarie pentru a comunica cu modulul de RTC
#include <Wire.h> // Librarie pentru comunicatiile cu I2C
#include <LiquidCrystal_I2C.h> // Librarie pentru facilizarea afisarii datelor pe ecranul LCD
#include <OneWire.h> // Librarie pentru facilizarea comunicarii cu senzorul de temperatura
#include <DallasTemperature.h> // Librarie specifica senzorului de temperatura pe care l-am folosit
#include <IRremote.hpp> // Librarie pentru decodarea semnalelor primite de la senzorul IR

// LED-uri RGB
int ledRGB_R = 6; // pinul 6 (pwm)
int ledRGB_G = 5; // pinul 7 (pwm)
int ledRGB_B = 3; // pinul 3 (pwm)

// LED-uri albe
int led_white = 10; // pinul 10 (pwm)
int is_on = 1; // Variabila pentru a porni/opri ledurile

// LCD I2C
LiquidCrystal_I2C lcd(0x27, 16, 2); // (adresa I2C, nr coloane, nr linii)

// Modul RTC
DS1307 rtc;

// Senzor IR
const int RECV_PIN = 7; // pinul 7
String code = ""; // // Variabila pentru a stoca codul citit de senzor

// Senzor de temperatura DS18B20
int temp = 2; // pinul 2
OneWire oneWire(temp); // Instanta OneWire pentru a comunica cu device-uri OneWire
DallasTemperature temp_sensor(&oneWire); // Pasez adresa instantei 

void setup() {
  // Declar pinii de la led-urile RGB ca fiind output:
  pinMode(ledRGB_R, OUTPUT);
  pinMode(ledRGB_G, OUTPUT);
  pinMode(ledRGB_B, OUTPUT);

  // Declar pin-ul led-urilor albe ca fiind output
  pinMode(led_white, OUTPUT);

  // Setare LED-uri RGB ca fiind stinse initial
  digitalWrite(ledRGB_R, 1);
  digitalWrite(ledRGB_G, 1);
  digitalWrite(ledRGB_B, 1);

  // Setare LED-uri albe ca fiind aprinse initial
  digitalWrite(led_white, 0);

  // Inițializare LCD
  lcd.init();
  lcd.backlight();
  lcd.print("Ora:");
  lcd.setCursor(0, 1);
  lcd.print("Temperatura:");
  lcd.setCursor(15, 1);
  lcd.print("C");

  // Pornire si setare modul RTC
  rtc.begin();
  rtc.set(0, 9, 15, 23, 05, 2024); //sec, min, hour, day, month, year
  rtc.start();

  Serial.begin(9600);

  // Pornire citire date de pe senzorul IR
  IrReceiver.begin(RECV_PIN, DISABLE_LED_FEEDBACK);
  
  // Pornire senzor de temperatura
  temp_sensor.begin();
}

void loop() {
  // Variabile pentru a retine ora si data
  uint8_t sec, min, hour, day, month;
  uint16_t year;

  // Obtin datele despre ora si data de la senzorul RTC
  rtc.get(&sec, &min, &hour, &day, &month, &year);

  // Afișează ora actuală pe LCD
  lcd.setCursor(5, 0);  // Setează cursorul la linia 1, poziția 5
  if (hour < 10) {
    // Daca ora are un singur caracter, mai adaugam noi un "0".
    lcd.print("0");
    lcd.print(hour, DEC);
  } else {
    lcd.print(hour, DEC);
  }

  lcd.print(':');
  if (min < 10) {
    lcd.print("0");
    lcd.print(min, DEC);
  } else {
    lcd.print(min, DEC);
  }

  lcd.print(':');
  if (sec < 10) {
    lcd.print("0");
    lcd.print(sec, DEC);
  } else {
    lcd.print(sec, DEC);
  }

  // Obtinere temperatura de la senzor si afisare pe ecran
  temp_sensor.requestTemperatures(); 
  lcd.setCursor(12, 1);
  lcd.print(temp_sensor.getTempCByIndex(0));

  // Citesc nivelul de lumina de la pin-ul A0
  int value = analogRead(A0);
 
 // Daca LED-urile albe sunt aprinse, li se regleaza intensitatea in functie de lumina
  if (is_on == 1) {
    if (value < 150) { // Daca este lumina slaba in jur
      analogWrite(led_white, 250); // LED-ul lumineaza mai intens
    } else if (value >= 150 && value < 300) {
      analogWrite(led_white, 200);
    } else if (value >= 300 && value < 500) {
      analogWrite(led_white, 150);
    } else if (value >= 500 && value < 700) {
      analogWrite(led_white, 100);
    } else {
      analogWrite(led_white, 50);
    }
  } else {
    analogWrite(led_white, 0);
  }

  // Citire si decodare date de pe senzor
  if (IrReceiver.decode()) {
    code = String(IrReceiver.decodedIRData.decodedRawData, HEX);
    if (code == "fb04ef00") { // rosu
      digitalWrite(ledRGB_R, 0);
      digitalWrite(ledRGB_G, 1);
      digitalWrite(ledRGB_B, 1);
    } else if (code == "fa05ef00") { // verde
      digitalWrite(ledRGB_R, 1);
      digitalWrite(ledRGB_G, 0);
      digitalWrite(ledRGB_B, 1);
    } else if (code == "f906ef00") { // albastru
      digitalWrite(ledRGB_R, 1);
      digitalWrite(ledRGB_G, 1);
      digitalWrite(ledRGB_B, 0);
    } else if (code == "f708ef00") { // portocaliu
      analogWrite(ledRGB_R, 0);
      analogWrite(ledRGB_G, 200);
      analogWrite(ledRGB_B, 255);
    } else if (code == "f609ef00") { // galben
      analogWrite(ledRGB_R, 0);
      analogWrite(ledRGB_G, 120);
      analogWrite(ledRGB_B, 255);
    } else if (code == "f40bef00") { // mov
      analogWrite(ledRGB_R, 0);
      analogWrite(ledRGB_G, 255);
      analogWrite(ledRGB_B, 0);
    } else if (code == "f50aef00") { // turn OFF LED-uri RBG 
      digitalWrite(ledRGB_R, 1);
      digitalWrite(ledRGB_G, 1);
      digitalWrite(ledRGB_B, 1);
    } else if (code == "fc03ef00") { // turn OFF LED-uri albe
      is_on = 0;
    } else if (code == "fd02ef00") { // turn ON LED-rui albe
      is_on = 1;
    }

    IrReceiver.resume();
  }

  // delay(500);
}